import { h, Host } from "@stencil/core";
const REGEX_TO_TEST_DIP = /^\d+(dip)?$/;
const REGEX_TO_TEST_PERCENTAGE = /^\d+(%)?$/;
const REGEX_TO_REPLACE_PERCENTAGE = /%/g;
export class GxIntersectionObserver {
  constructor() {
    this.defaultThreshold = [0];
    this.rootMarginString = "";
  }
  /**
   * Used to check that the passed value is a valid Device-independent-pixel (dip) value, and transform the dip to px value @example: "200dip", "50dip"
   * @param percentValue : Represent the given dip value
   * @returns When is a valid dip value returns the value converted to pixels, otherwise returns "0px"
   */
  checkValidDipValue(dipValue) {
    return REGEX_TO_TEST_DIP.test(dipValue)
      ? this.convertDipToPxValue(dipValue)
      : "0px";
  }
  /**
   * Used to check that the passed value is a valid percentage. @example: "200%"
   * @param percentValue : Represent the given percentage value
   * @returns When is a valid percentage value returns the value, otherwise returns "0px"
   */
  checkValidPercentValue(percentValue) {
    return REGEX_TO_TEST_PERCENTAGE.test(percentValue) ? percentValue : "0px";
  }
  /**
   * Transform the given dip value to pixels
   * @param dipValue : Represent the given dip value @example "10dip"
   * @returns Returns the given value converted to px @example "10px"
   */
  convertDipToPxValue(dipValue) {
    const aux = dipValue.replace("dip", "px");
    return aux.split(" ").join("");
  }
  /**
   * Convert the given percentages string representing a threshold value in decimal value
   * @param threshold Threshold string representation @example: "10%"
   * @returns Decimal value between 0 and 1 @example: 0.1
   */
  convertThresholdValueToNumber(threshold) {
    if (REGEX_TO_TEST_PERCENTAGE.test(threshold)) {
      return Number(threshold.replace(REGEX_TO_REPLACE_PERCENTAGE, "")) / 100;
    }
    return 0;
  }
  /**
   * Create a valid Javascript IntersectionObserver API threshold  options value
   * @param threshold Threshold string representation @example: "10%,30%,40%"
   * @returns Valid array of numbers in the range between 0 and 1 @example: [0.1, 0.3, 0.4]
   */
  parseThreshold(threshold) {
    if (!threshold) {
      return [0];
    }
    const finalThresholds = [];
    const thresholdsToParse = threshold.split(",");
    thresholdsToParse.forEach(thresholdValue => {
      const convertedNumber = this.convertThresholdValueToNumber(thresholdValue);
      if (convertedNumber <= 1) {
        finalThresholds.push(convertedNumber);
      }
    });
    return finalThresholds;
  }
  /**
   * Initialize intersection observer with its options and start observing
   */
  setIntersectionObserver() {
    const options = {
      root: this.rootElement,
      rootMargin: this.rootMarginString,
      threshold: this.defaultThreshold
    };
    this.observer = new IntersectionObserver(entries => {
      const detail = entries[0];
      this.intersectionUpdate.emit(detail);
    }, options);
    this.observer.observe(this.element);
  }
  /**
   * Transform the given initial intersection-observer component properties into Javascript IntersectionObserver API options valid values
   */
  setIntersectionObserverOptionsFromProperties() {
    if (this.root) {
      this.rootElement = document.getElementById(this.root);
    }
    this.rootMarginString = [
      this.validatePosition(this.topMargin),
      this.validatePosition(this.leftMargin),
      this.validatePosition(this.bottomMargin),
      this.validatePosition(this.rightMargin)
    ].join(" ");
    this.defaultThreshold = this.parseThreshold(this.threshold);
  }
  /**
   * Check that the given string value is a valid percentage or device independent pixel (dip) value
   * @param position : Represent the given dip or px value
   */
  validatePosition(position) {
    if (position && position.endsWith("dip")) {
      return this.checkValidDipValue(position);
    }
    return this.checkValidPercentValue(position);
  }
  componentDidLoad() {
    this.setIntersectionObserverOptionsFromProperties();
    this.setIntersectionObserver();
  }
  disconnectedCallback() {
    if (this.observer !== undefined) {
      this.observer.disconnect();
      this.observer = undefined;
    }
  }
  render() {
    return (h(Host, { class: {
        [this.cssClass]: !!this.cssClass
      } }, h("slot", { name: "content" })));
  }
  static get is() { return "gx-intersection-observer"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() {
    return {
      "$": ["gx-intersection-observer.css"]
    };
  }
  static get styleUrls() {
    return {
      "$": ["gx-intersection-observer.css"]
    };
  }
  static get properties() {
    return {
      "bottomMargin": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Bottom margin around the root element"
        },
        "attribute": "bottom-margin",
        "reflect": false
      },
      "cssClass": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "A CSS class to set as the gx-intersection-observer element class"
        },
        "attribute": "css-class",
        "reflect": false
      },
      "leftMargin": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Left margin around the root element"
        },
        "attribute": "left-margin",
        "reflect": false
      },
      "threshold": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Numeric values representing percentages of the target element which are visible."
        },
        "attribute": "threshold",
        "reflect": false
      },
      "topMargin": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Top margin around the root element"
        },
        "attribute": "top-margin",
        "reflect": false
      },
      "rightMargin": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [],
          "text": "Right margin around the root element"
        },
        "attribute": "right-margin",
        "reflect": false
      },
      "root": {
        "type": "string",
        "mutable": false,
        "complexType": {
          "original": "string",
          "resolved": "string",
          "references": {}
        },
        "required": false,
        "optional": false,
        "docs": {
          "tags": [{
              "name": "Prop",
              "text": "() Set the ID of the component that is used as the viewport, default is the browser."
            }],
          "text": ""
        },
        "attribute": "root",
        "reflect": false
      }
    };
  }
  static get events() {
    return [{
        "method": "intersectionUpdate",
        "name": "intersectionUpdate",
        "bubbles": true,
        "cancelable": true,
        "composed": true,
        "docs": {
          "tags": [{
              "name": "param",
              "text": "IntersectionObserverEntry : Details of intersection object."
            }],
          "text": "Emitted whenever the control reaches a threshold specified by the threshold property"
        },
        "complexType": {
          "original": "IntersectionObserverEntry",
          "resolved": "IntersectionObserverEntry",
          "references": {
            "IntersectionObserverEntry": {
              "location": "global"
            }
          }
        }
      }];
  }
  static get elementRef() { return "element"; }
}
